@@ -237,14 +237,14 @@ static boolean_t _send(List_T list) {
                                                 "MIME-Version: 1.0\r\n"
                                                 "Content-Type: text/plain; charset=utf-8\r\n"
                                                 "Content-Transfer-Encoding: 8bit\r\n"
-                                                "Message-Id: <%lld.%lu@%s>\r\n"
+                                                "Message-Id: <%lld.%"PRIx64"@%s>\r\n"
                                                 "\r\n"
                                                 "%s",
                                                 m->to,
                                                 m->subject,
                                                 now,
                                                 VERSION,
-                                                (long long)Time_now(), random(), Run.mail_hostname ? Run.mail_hostname : Run.system->name,
+                                                (long long)Time_now(), System_randomNumber(), Run.mail_hostname ? Run.mail_hostname : Run.system->name,
                                                 m->message) <= 0
                                    )
                                 {
