@@ -783,8 +783,10 @@ char *Ssl_printOptions(SslOptions_T options, char *b, int size) {
         *b = 0;
         if (options->flags) {
                 int count = 0;
-                if (options->version != -1)
-                        snprintf(b + strlen(b), size - strlen(b) - 1, "%sversion: %s", count++ ? ", " : "", sslnames[options->version]);
+                if (options->version != -1) {
+                        snprintf(b + strlen(b), size - strlen(b) - 1, "version: %s", sslnames[options->version]);
+                        count++;
+                }
                 if (options->verify == true)
                         snprintf(b + strlen(b), size - strlen(b) - 1, "%sverify: enable", count++ ? ", " : "");
                 if (options->allowSelfSigned == true)
